Rem
	Demolition Derby AI Code
EndRem

Const Weakling = 1
Const Moderate = 2
Const Offensive = 3

Const NullState = 0
Const WanderState = 1
Const EvadeState = 2
Const SeekHealthState = 3
Const SeekGunState = 4
Const KillState = 5

Type AI Extends ControlObject
	Field GoalX:Float
	Field GoalY:Float
	Field PickUp:HealthPickUp
	Field hDesire:Int 'AI desire for health
	Field kDesire:Int 'AI desire for kill
	Field eDesire:Int 'AI desire for evading
	Field wDesire:Int 'AI desire for wander
	Field gDesire:Int 'AI desire for gun
	Field target:Vehicle
	Field DesiredWeapon:Int
	Field RespawnTime:Int
	Field checkVal:Int
	
	Global PotentialNames:String[] = ["Mario","Bob","Thomas","Rudy","Andrew","Alex","Sarah","Lisa","Michael","Chris","Rebecca","Andy","Freud","Amanda","John McClain","Angela","Daniel","Don", "Joe", "Mitch", "Natalie", "Stephen", "Brian", "Wheelz", "Josh", "TJ"]
	
	' SDAI Related Fields
	Field currentState:State
	Field currentStateID:Int
	Field previousState:State
	Field globalState:State
	
	Global List:TList
	
	Function Make:AI(Style=1,Team=0)
		Local tAI:AI = New AI
		If List = Null List = CreateList()
		List.AddLast tAI
		ControlObject.List.AddLast tAI
		
		tAI.Style = Style
		
		If Team = 5
			Team = 0
		Else
			tAI.Team = Team
		EndIf
		
		Local myPoint:Int = Rnd(1,RespawnPoint.List.Count()+1)
		Local tempVal:Int
		
		For Local tSpawnPoints:RespawnPoint = EachIn RespawnPoint.List
			tempVal:+1
			If tempVal = myPoint
				Local spawnX = tSpawnPoints.GetX()
				Local spawnY = tSpawnPoints.GetY()
				Print spawnx
				Print spawny
			EndIf
		Next
		
		Select Style
			Case Weakling
				tAI.DesiredWeapon = 0
				tempVeh = Rnd(1,3)
			Case Moderate
				tAI.DesiredWeapon = 3
				tempVeh = Rnd(1,5)
			Case Offensive
				tAI.DesiredWeapon = 4
				tempVeh = Rnd(1,6)
		EndSelect
		
		Select tempVeh
			Case 1
				tAI.myVehicle = Jeep.Spawn(SpawnY,SpawnX,0,tAI)
			Case 2
				tAI.myVehicle = Car.Spawn(SpawnY,SpawnX,0,tAI)	
			Case 3
				tAI.myVehicle = SportsCar.Spawn(SpawnY,SpawnX,0,tAI)
			Case 4
				tAI.myVehicle = ConceptCar.Spawn(SpawnY,SpawnX,0,tAI)
			Case 5
				tAI.myVehicle = SemiTruck.Spawn(SpawnY,SpawnX,0,tAI)
		EndSelect
		
		tAI.myVehicle.TreadCap = 5
		tAI.Target = Null
		tAI.Points = 0
		tAI.myMapDot = MinimapDot.Create(miniMapHUD,tAI)
		tAI.Name = PotentialNames[Rnd(0,PotentialNames.Length)]
		
		Select Team
			Case TEAM_ONE
				tAI.myVehicle.mySprite.tintRed = 255
				tAI.myVehicle.mySprite.tintBlue = 0
				tAI.myVehicle.mySprite.tintGreen = 0
			Case TEAM_TWO
				tAI.myVehicle.mySprite.tintRed = 0
				tAI.myVehicle.mySprite.tintBlue = 0
				tAI.myVehicle.mySprite.tintGreen = 255	
			Case TEAM_THREE
				tAI.myVehicle.mySprite.tintRed = 0
				tAI.myVehicle.mySprite.tintBlue = 255
				tAI.myVehicle.mySprite.tintGreen = 0
			Case TEAM_FOUR
				tAI.myVehicle.mySprite.tintRed = 255
				tAI.myVehicle.mySprite.tintBlue = 0
				tAI.myVehicle.mySprite.tintGreen = 255		
		EndSelect
		
		tAI.currentStateID = WanderState
		tAI.currentState = Wander.Instance()
		
		ConsoleMessage.Create("["+CurrentTime$()+"] AI added to the scene [AI Position: "+tAI.myVehicle.X+"/"+tAi.myVehicle.Y+"]",INFO_MSG)
		
		Return tAI
	EndFunction
	
	Method Update()
	
		' If the AI's vehicle dies, handle respawn
		If myVehicle.Dead = True
			ChangeState(Nothing.Instance())
			Respawn()
		EndIf
	
		' Handle Vehicle tread
		myVehicle.TreadWait:+1
		If myVehicle.TreadWait => myVehicle.TreadCap
			Tread.Make(myVehicle.X,myVehicle.Y,-myVehicle.Angle)
			myVehicle.TreadWait = 0
		EndIf
	
		' Weigh the actions and make a decision
		WeighActions()
		
		' Search for any nearby targets
		searchForTarget()

		' Shoot the selected target
		Shoot()
	
		' Execute the current AI State
		currentState.ExecuteState(Self)	
		
		SetAlpha 100
		SetScale 1,1
		SetColor 255,255,255
		
		' Update the Vehicle
		myVehicle.Update()
		
		' If the AI wanders out of the playing field accidentaly, write a report to the
		' debug console and respawn the AI inside of the playing field. 
		If myVehicle.X > width Or myVehicle.X < 0 Or myVehicle.Y > height Or myVehicle.X < 0
			ConsoleMessage.Create("Error: AI vehicle out of map boundaries!",ERROR_MSG)
			ConsoleMessage.Create("Report: AI respawned due to positional error on map!",SYS_MSG)
			ChangeState(Nothing.Instance())
			Respawn()
		EndIf
			
	EndMethod
	
	Method Respawn()
		
		' Some required local variables
		Local myPoint:Int = Rnd(1,RespawnPoint.List.Count()+1)
		Local tempX:Int
		Local point:RespawnPoint
		
		' Clean stuff up!
		If myVehicle.WeaponSlot <> Null
			myVehicle.WeaponSlot.Destroy()
		EndIf
		
		If myVehicle.Weaponslot2 <> Null
			myVehicle.WeaponSlot.Destroy()
		EndIf
			
		
		' Find the randomly selected respawn point
		For Local tSpawnPoints:RespawnPoint = EachIn RespawnPoint.List
			tempX:+1
			If tempX = myPoint
				Local spawnX = tSpawnPoints.GetX()
				Local spawnY = tSpawnPoints.GetY()
				point = tSpawnPoints
				point.Active = True
			EndIf
		Next
		
		' Reset Various Variables :)
		myVehicle.mySprite.SetPosition(spawnX,spawnY)
		myVehicle.Health = 100
		myVehicle.WeaponSlot = MachineGun.Choose(myVehicle)
		myVehicle.Dead = False
		
		point.Active = False
		
		' Send a message to the debug console so we know what's going on at this point
		ConsoleMessage.Create("["+CurrentTime$()+"] AI added to the scene [AI Position: "+myVehicle.X+"/"+myVehicle.Y+"]",INFO_MSG)
		
		' Set the AI state to the default.
		ChangeState(Wander.Instance())
	EndMethod
	
	Method ChangeState(newState:State)
	
		' keep track of the previous AI state
		previousState = currentState
	
		currentState.ExitState(Self)
		currentState = newState
		currentState.EnterState(Self)
		
	EndMethod
	
	Method RevertToPreviousState()

		ChangeState(previousState)
	EndMethod
	
	Method isInState:Int(checkState:Int)
		If currentStateID = checkState
			Return True
		EndIf
		Return False
	EndMethod
	
	Method WeighActions:Int()

		checkWander()
		checkHealth()
		checkGun()
		checkKill()
		
		decideAction()
		
	EndMethod
	
	Method Shoot()
	
		If Target.Dead = True
			Target = Null
			Return
		EndIf
		
		If Distance(myVehicle.X,myVehicle.Y,Target.X,Target.Y) < 650
			Local tempValX:Float = Target.X - (myVehicle.X)
			Local tempValY:Float = Target.Y - (myVehicle.Y)
			
			Local Angle:Float = ATan2(tempValY,tempValX)
			
			Select Style
				Case Weakling
					Angle:+AddSubtract(Rnd(15))
				Case Moderate
					Angle:+AddSubtract(Rnd(10))
				Case Offensive
					Angle:+AddSubtract(Rnd(3))
			EndSelect
			
			myVehicle.WeaponSlot.Angle = Angle
			
			If myVehicle.WeaponSlot.Ammo = -1 Or myVehicle.WeaponSlot.Ammo > 0
				myVehicle.WeaponSlot.Emitter.Draw()
				myVehicle.ShootWait:+1
				If myVehicle.ShootWait => myVehicle.WeaponSlot.RateofFire
					If myVehicle.WeaponSlot.Ammo = -1
						myVehicle.WeaponSlot.ShotType.Fire( myVehicle.WeaponSlot.X, myVehicle.WeaponSlot.Y, myVehicle.WeaponSlot.Angle,Self)
						myVehicle.ShootWait = 0
					EndIf
					If myVehicle.WeaponSlot.Ammo > 0
						myVehicle.WeaponSlot.ShotType.Fire( myVehicle.WeaponSlot.X, myVehicle.WeaponSlot.Y, myVehicle.WeaponSlot.Angle,Self)
						myVehicle.ShootWait = 0
						myVehicle.WeaponSlot.Ammo:-1
					EndIf					
				EndIf
			EndIf
		EndIf		
	EndMethod
	
	Method searchForTarget()
		
		Local tempVal:Float = 9999
		For Local tVehicle:Vehicle = EachIn Vehicle.List
			If tVehicle <> myVehicle
				If tVehicle.Parent.Team <> Team Or Team = 0
					If tempVal > Distance(myVehicle.X,myVehicle.Y,tVehicle.X,tVehicle.Y)
						tempVal = Distance(myVehicle.X,myVehicle.Y,tVehicle.X,tVehicle.Y)
						Target = tVehicle
					EndIf
				EndIf
			EndIf
		Next
		
	EndMethod
	
	Method decideAction()
	
		'Print "Wander Desire: "+wDesire
		'Print "Health Desire: "+hDesire
		'Print "Gun Desire: "+gDesire
		'Print "Kill Desire: "+kDesire
		
		Local desireArray:Int[] = [wDesire, hDesire, gDesire, kDesire]
		
		Local highestVal = FindHighest(desireArray)

		Select highestVal
			Case wDesire
				If Not isInState(WanderState) Then ChangeState(Wander.Instance())
			Case hDesire
				If Not isInState(SeekHealthState) Then ChangeState(SeekHealth.Instance())
			Case gDesire
				If Not isInState(SeekGunState) Then ChangeState(SeekGun.Instance())
			Case kDesire
				If Not isInState(KillState) Then ChangeState(Kill.Instance())
		EndSelect
		
	EndMethod
	
	Method checkKill()
		Select Style
			Case 1
				kDesire = 15
			Case 2
				kDesire = 23
			Case 3
				kDesire = 29
		EndSelect
		
		If kDesire > 100 Then kDesire = 100
	EndMethod
	
	Method checkGun()
		If DesiredWeapon = 0
			gDesire = 20
			Return
		EndIf
		
		If myVehicle.WeaponSlot.Number < DesiredWeapon
			gDesire = 20 + ((DesiredWeapon - myVehicle.WeaponSlot.Number)*5)
		Else
			gDesire = 20
		EndIf
		
		If gDesire > 100 Then gDesire = 100
		If gDesire < 0 Then gDesire = 0
		
	EndMethod
	
	Method checkHealth()

		hDesire = 75 - myVehicle.Health 		
		
		Select Style
			Case 1
				hDesire:+15
			Case 3 
				hDesire:-15
			Default
				hDesire = hDesire
		EndSelect
		
		If hDesire > 75 Then hDesire = 75
		If hDesire < 20 Then hDesire = 20
		
	EndMethod
	
	Method checkWander()
		wDesire = 25
	EndMethod

	Function UpdateAll()
	
		If Not list Return
		
		Local tAI:AI
		For tAI = EachIn List
			tAi.Update()
		Next
		
	EndFunction
	
	Function DestroyAll()
		If Not List Return
		
		Local tAI:AI
		For tAI = EachIn List
			tAI.Destroy()
		Next
	EndFunction
	
	Method Destroy()
		List.Remove(Self)
	EndMethod
	
EndType
		
		