Import PUB.Mappy
Import SAS.spritebehaviors
Import Fry.FryGUI

AppTitle = "Demolition Derby v1.0 [RC1]"

Include "gameobject.bmx"
Include "vehicle.bmx"
Include "weapon.bmx"
Include "shot.bmx"
Include "tread.bmx"
Include "pickups.bmx"
Include "ai.bmx"
Include "sdailib.bmx"
Include "explosion.bmx"
Include "secondaryweapons.bmx"
Include "wrecks.bmx"
Include "respawnpoint.bmx"
Include "player.bmx"
Include "debugconsole.bmx"
Include "musicplayer.bmx"
Include "gui.bmx"
Include "configfiles.bmx"
Include "minimap.bmx"
Include "matchtimer.bmx"
Include "mapload.bmx"

SeedRnd MilliSecs()

'Configuration File Loadup
Global Config:TConfig = New TConfig
Config.Name = "config"
Config.Init()

formatCheck = Float(Config.Read("Screen Format","0"))
sfxVolume:Float = Float(Config.Read("Sound Volume","5"))/10
soundVolume:Float = Float(Config.Read("Music Volume","10"))/10
Print sfxVolume
Print soundVolume

Select formatCheck
	Case 0
		gfxwidth = 1024
		gfxheight = 768
	Case 1
		gfxwidth = 1280
		gfxheight = 720
EndSelect

Graphics 1024,768,32

SetMaskColor 255,0,255
Global px:Int
Global py:Int

' Game State Stuff
Global GameState:Int
Const MAINMENU 			= 1
Const BATTLESETUP			= 2
Const GAMESETUP			= 3
Const PLAYING				= 4

GameState = MAINMENU

'Game Type Stuff
Const DEATHMATCH			= 1
Const BIGDOG				= 2
Const ASSAULT				= 3
Const CTF					= 4

' Graphics Quality Stuff
Global GRAPHIC_QUALITY =  GetGraphicsQuality()
Const GFX_HIGH 		 = 2
Const GFX_MED			= 1
Const GFX_LOW			= 0

' Match Related Globals/Constants
Global MatchPoint:Int
Global MatchType:Int
Global MatchTime:Int
Global MatchRespawn:Int

' SFX Related Globals
Global mgFire:TSound
Global hmgFire:TSound
Global rlFire:TSound
Global glFire:TSound
Global explode:TSound
Global mgChannel:TChannel = AllocChannel()
Global hmgChannel:TChannel = AllocChannel()
Global rlChannel:TChannel = AllocChannel()
Global glChannel:TChannel = AllocChannel()
Global explodeChannel:TChannel = AllocChannel()
Global phaseFire:TSound
Global phaseChannel:TChannel = AllocChannel()
SetChannelVolume(explodeChannel,sfxVolume)
SetChannelVolume(mgChannel,sfxVolume)
SetChannelVolume(hmgChannel,sfxVolume)
SetChannelVolume(rlChannel,sfxVolume)
SetChannelVolume(glChannel,sfxVolume)
SetChannelVolume(phaseChannel,sfxVolume)
SetChannelVolume(MUSIC_CHANNEL,soundVolume)


' Map/Font Stuff
Global map:MappyMap = New MappyMap
Global hud_font:TImagefont=LoadImageFont("C:\windows\fonts\tahoma.ttf",25,SMOOTHFONT)
Global respawn_font:TImageFont=LoadImageFont("C:\windows\fonts\tahoma.ttf",72,SMOOTHFONT)
Global debug_font:TImageFont=LoadImageFont("C:\windows\fonts\tahoma.ttf",12,SMOOTHFONT)
Global plumeImage = LoadImage("sprites/gradball.png")
Global targetImage = LoadImage("Sprites/target.png")
Local logoImage = LoadImage("sprites/backdrop/logo.png")
Local mainMenuImage = LoadImage("sprites/backdrop/mainmenu.png")
Global bigDogImage = LoadImage("sprites/bigdog.png")
MidHandleImage(bigDogImage)
Global miniMapHUD:Minimap

Global TestPlayer:Player

SetImageFont respawn_font
DrawImage logoImage,0,0
SetColor 0,0,0
DrawText "LOADING...",GraphicsWidth()/2-(TextWidth("LOADING...")/2),GraphicsHeight()/2-(TextHeight("LOADING..")/2-100)
SetColor 255,255,255
SetImageFont hud_font

Flip

CreatePlayList()
LoadSFX()

Global world:GameWorld

Global width:Double
Global height:Double
Global midX:Double
Global midY:Double

HideMouse
InitializeGUI()
Delta.Start()
Timer.Reset()

Repeat

	SetScale 1,1
	SetAlpha 100
	SetColor 255,255,255
	SetBlend(AlphaBlend)

	Select GameState
		
		Case MAINMENU
			DrawImage mainMenuImage,0,0

			DrawText "copyright (c)2007 outboxgames - http://www.outboxgames.com",0,GraphicsHeight()-TextHeight("7")
			
			PollSystem
	
			fry_Refresh()
			
			CheckGUIEvents()
	
			fry_FPS.Update()
		
		Case PLAYING
		
			HandleScores()
	
			px = (TestPlayer.myVehicle.X)-(GraphicsWidth()/2)
			py = (TestPlayer.myVehicle.Y)-(GraphicsHeight()/2)
			If px > (map.mapwidth*map.mapblockwidth-GraphicsWidth()) Then px = map.mapwidth*map.mapblockwidth-GraphicsWidth()
			If py > (map.mapheight*map.mapblockheight-GraphicsHeight()) Then py = map.mapheight*map.mapblockheight-GraphicsHeight()
			If px < 0 Then px = 0
			If py < 0 Then py = 0

			map.MapDrawBG(px, py, 0, 0, GraphicsWidth(), GraphicsHeight())
	
			Scar.UpdateAll()
			Tread.UpdateAll()
			Wreckage.UpdateAll()
			world.Render()
			SetColor 255,255,255 'Reset
			Shot.UpdateSecondary()
			HealthPickUp.UpdateAll()
			WeaponPickUp.UpdateAll()
			SecondaryPickUp.UpdateAll()
			AI.UpdateAll()
			Player.UpdateAll()
			Shot.UpdatePrimary()
			Weapon.UpdateAll()
			Explosion.UpdateAll()
			DrawHud()
			Minimap.UpdateAll()
			ConsoleMessage.Update()
			Timer.Update()
			
			PlayerControls()
		
			world.Update(Delta.Time())
			
	EndSelect
	
	PlayMusic()
			
	Delta.Update()
	Flip
	Cls
Until KeyDown(Key_escape)

Function HandleScores()

	'Check AI Scores to see if they match win-score
	If AI.List
		For Local tAI:AI = EachIn AI.List
			If tAI.Points => MatchPoint
				GetMatchResults()
				PerformCleanUp()
				GameState = MAINMENU
				fry_SetScreen("Match Results")
				Return
			EndIf
		Next
	EndIf
	
	'Check Player Scpres
	If Player.List
		For Local tPlayer:Player = EachIn Player.List
			If tPlayer.Points => MatchPoint
				GetMatchResults()
				PerformCleanup()
				GameState = MAINMENU
				fry_SetScreen("Match Results")
				Return
			EndIf
		Next
	EndIf
	
	' if Timer runs out
	If MatchTime <= 0
		GetMatchResults()
		PerformCleanup()
		GameState = MAINMENU
		fry_SetScreen("Match Results")
		Return
	EndIf
	
EndFunction

Function GetMatchResults()
	
	Local totalPlayers:Int = AI.List.Count()+1
	Local scoreBoard:ControlObject[totalPlayers]
	Local x:Int = 0
	
	For Local ttAI:AI = EachIn AI.List
		scoreBoard[x] = ttAI
		x:+1
	Next
	
	scoreBoard[x] = TestPlayer
	
	SortArray(scoreBoard)
	

EndFunction

Function SortArray:ControlObject[](array:ControlObject[],timesToSort:Int=5)
	Local oVal:ControlObject
	Local sVal:ControlObject
	
	For Local y = 1 To timesToSort
		For Local x:Int = 0 To (array.length-1)
			If (x+1) < array.length
				If array[x+1].Points > array[x].Points
					oVal = array[x]
					sVal = array[x+1]
					array[x] = sVal
					array[x+1] = oVal
				EndIf
			EndIf
		Next
	Next
	
	For x = 0 To (array.Length-1)
		resultsTable.AddItem([array[x].Name, String(array[x].Points), String(array[x].deaths), String(array[x].Points)])
	Next
EndFunction

Function PerformCleanup()
	AI.DestroyAll()
	Player.DestroyAll()
	Vehicle.DestroyAll()
	Shot.DestroyAll()
	miniMapHUD.Destroy()
	Wreckage.DestroyAll()
	MinimapDot.DestroyAll()
	world.ClearWalls()
	Tread.DestroyAll()
	WeaponPickUp.DestroyAll()
	SecondaryPickUp.DestroyAll()
	HealthPickUp.DestroyAll()
	Explosion.DestroyAll()
	Scar.DestroyAll()
EndFunction

Function HandleTimer()
EndFunction

Function PlayerControls()
	If KeyHit(KEY_TILDE)
		If DEBUG_DISPLAY = False
			FlushKeys()
			DEBUG_DISPLAY = True
			world.ToggleShowFPS()
		Else
			FlushKeys()
			DEBUG_DISPLAY = False
			world.ToggleShowFPS()
		EndIf
	EndIf
	
	If DEBUG_DISPLAY = False
		If KeyDown(KEY_UP) Or KeyDown(KEY_W) Then TestPlayer.myVehicle.Up = True Else TestPlayer.myVehicle.Up = False
		If KeyDown(KEY_DOWN) Or KeyDown(KEY_S) Then TestPlayer.myVehicle.Down = True Else TestPlayer.myVehicle.Down = False
		If KeyDown(KEY_LEFT) Or KeyDown(KEY_A) Then TestPlayer.myVehicle.LeftTurn = True Else TestPlayer.myVehicle.LeftTurn = False
		If KeyDown(KEY_RIGHT) Or KeyDown(KEY_D) Then TestPlayer.myVehicle.RightTurn = True Else TestPlayer.myVehicle.RightTurn = False
		If MouseDown(MOUSE_LEFT) Then TestPlayer.myVehicle.Shoot = True Else TestPlayer.myVehicle.Shoot = False
		If MouseHit(MOUSE_RIGHT) Then TestPlayer.myVehicle.Shoot2 = True Else TestPlayer.myVehicle.Shoot2 = False
		DrawText "Score: "+TestPlayer.Points,10,10
	EndIf	
	
	DrawImage targetImage,MouseX(),MouseY()

EndFunction

Function DrawHUD()
	SetScale 1,1
	SetBlend AlphaBlend
	SetAlpha 0.5
	SetImageFont hud_font
	If TestPlayer.myVehicle.Health > 15
		SetColor 255,255,255
	Else
		SetColor 255,0,0
	EndIf
	DrawRect 0,GraphicsHeight()-36,GraphicsWidth(),GraphicsHeight()
	SetColor 0,0,0
	SetAlpha 1
	If TestPlayer.myVehicle.WeaponSlot.Ammo <> - 1
		DrawText ""+TestPlayer.myVehicle.WeaponSlot.Name+" | "+TestPlayer.myVehicle.WeaponSlot.Ammo,10,GraphicsHeight()-36
	Else
		DrawText ""+TestPlayer.myVehicle.WeaponSlot.Name+" | Unlimited",10,GraphicsHeight()-36
	EndIf
	
	DrawText "Health: "+TestPlayer.myVehicle.Healthreport,GraphicsWidth()/2-(TextWidth("Health: "+TestPlayer.myVehicle.Healthreport)/2),GraphicsHeight()-36
	
	If TestPlayer.myVehicle.WeaponSlot2.Ammo > 0
		DrawText ""+TestPlayer.myVehicle.WeaponSlot2.Name+" | "+TestPlayer.myVehicle.WeaponSlot2.Ammo,GraphicsWidth()-(TextWidth(""+TestPlayer.myVehicle.WeaponSlot2.Name+" | "+TestPlayer.myVehicle.WeaponSlot2.Ammo))-5,GraphicsHeight()-36
	EndIf
	
	SetColor 255,255,255
EndFunction

'Give the distance between two points
Function Distance#( X1, Y1, X2, Y2 )
	Local DX# = X2 - X1
	Local DY# = Y2 - Y1
	Return Sqr(Dx*Dx + Dy*Dy)
EndFunction

Function Mean:Int(n1:Int=0,n2:Int=0,n3:Int=0)
	Local tempVal = n1+n2+n3
	Local mean = tempVal/3
	Return mean
EndFunction

Function AddSubtract:Float(val:Float)
	Local tempVal = Rnd(0,2)
	If tempVal = 0 Then Return -val
	If tempVal = 1 Then Return val
EndFunction

Function CheckGUIEvents()
	While fry_PollEvent()
		If fry_EventID() = fry_EVENT_GADGETSELECT
			
			Select fry_EventSource()
			
				Case startBattleB
					'SetDefaultMatch()
				Case optionsB
					SetOptionsValues()
				Case quitB
					End
				Case defaultOptionsB
					qualitySlider.SetValue(3)
					sfxSlider.SetValue(5)
					musicSlider.SetValue(9)
					nameField.SetText("DemoDerby Newbie")
				Case saveOptionsB
					SaveOptionsValues()
					GRAPHIC_QUALITY = GetGraphicsQuality()
				Case saveDefaultMatchB
					SaveDefaultMatch()
				Case defaultMatchB
					SetDefaultMatch()
				Case startMatchB
					If mapCombo.selectedItem() > -1 And vehicleCombo.selectedItem() > -1 And gameTypeCombo.selectedItem() > -1
						LaunchMatch()
						GameState = PLAYING
					EndIf
				Case continueMatchB
					resultsTable.ClearItems()
					
			EndSelect
			
		EndIf
	Wend
EndFunction

Function SetOptionsValues()
	qualitySlider.SetValue(Int(Config.Read("Graphics Quality","3")))
	sfxSlider.SetValue(Int(Config.Read("Sound Volume","5")))
	musicSlider.SetValue(Int(Config.Read("Music Volume","9")))
	nameField.SetText(Config.Read("Player Name","DemoDerby Newbie"))
EndFunction

Function SaveOptionsValues()
	Config.Write("Graphics Quality",String(qualitySlider.GetValue()))
	Config.Write("Sound Volume",String(sfxSlider.Getvalue()))
	config.Write("Music Volume",String(musicSlider.GetValue()))
	Config.Write("Player Name",nameField.getText())
EndFunction

Function SaveDefaultMatch()
	Config.Write("AI1Default",String(ai1combo.SelectedItem()+1))
	Config.Write("AI2Default",String(ai2combo.SelectedItem()+1))
	Config.Write("AI3Default",String(ai3combo.SelectedItem()+1))
	Config.Write("AI4Default",String(ai4combo.SelectedItem()+1))
	Config.Write("AI5Default",String(ai5combo.SelectedItem()+1))
	Config.Write("AI1TeamDefault",String(ai1teamcombo.SelectedItem()+1))
	Config.Write("AI2TeamDefault",String(ai2teamcombo.SelectedItem()+1))
	Config.Write("AI3TeamDefault",String(ai3teamcombo.SelectedItem()+1))
	Config.Write("AI4TeamDefault",String(ai4teamcombo.SelectedItem()+1))
	Config.Write("AI5TeamDefault",String(ai5teamcombo.SelectedItem()+1))
	Config.Write("MatchTypeDefault",String(gameTypeCombo.SelectedItem()+1))
	Config.Write("MapDefault",String(mapCombo.SelectedItem()+1))
	Config.Write("PlayerVehicleDefault",String(VehicleCombo.SelectedItem()+1))
	Config.Write("PlayerTeamDefault",String(plyrTeamCombo.SelectedItem()+1))
EndFunction

Function SetDefaultMatch()
	ai1combo.SelectItem(Int(Config.Read("AI1Default","-1"))-1)
	ai2combo.SelectItem(Int(Config.Read("AI2Default","-1"))-1)
	ai3combo.SelectItem(Int(Config.Read("AI3Default","-1"))-1)
	ai4combo.SelectItem(Int(Config.Read("AI4Default","-1"))-1)
	ai5combo.SelectItem(Int(Config.Read("AI5Default","-1"))-1)
	ai1teamcombo.SelectItem(Int(Config.Read("AI1TeamDefault","-1"))-1)
	ai2teamcombo.SelectItem(Int(Config.Read("AI2TeamDefault","-1"))-1)
	ai3teamcombo.SelectItem(Int(Config.Read("AI3TeamDefault","-1"))-1)
	ai4teamcombo.SelectItem(Int(Config.Read("AI4TeamDefault","-1"))-1)
	ai5teamcombo.SelectItem(Int(Config.Read("AI5TeamDefault","-1"))-1)
	gameTypeCombo.SelectItem(Int(Config.Read("MatchTypeDefault","-1"))-1)
	mapCombo.SelectItem(Int(Config.Read("MapDefault","-1"))-1)
	vehicleCombo.SelectItem(Int(Config.Read("PlayerVehicleDefault","-1"))-1)
	plyrTeamCombo.SelectItem(Int(Config.Read("PlayerTeamDefault","-1"))-1)
EndFunction

Function GetGraphicsQuality:Int()
	Local tempVal = Int(Config.Read("Graphics Quality","3"))
	Return tempVal
EndFunction

Function LaunchMatch()

	' Adds preliminary stuff to the Debug Console
	ConsoleMessage.Create("******************************************",INFO_MSG)
	ConsoleMessage.Create("Demolition Derby v0.2 ALPHA Debug Console",SYS_MSG)
	ConsoleMessage.Create("******************************************",INFO_MSG)
	
	ProcessMap(mapCombo.selectedText())
	
	miniMapHud = Minimap.Create(GraphicsWidth(),GraphicsHeight(),width,height,miniMapCheck.getState())

	TestPlayer = Player.Make(vehicleCombo.SelectedItem()+1, plyrTeamCombo.SelectedItem()+1)
	
	If ai1combo.SelectedItem() <> -1 And ai1combo.SelectedItem() < 3
		AI.Make(ai1combo.SelectedItem()+1,ai1teamcombo.SelectedItem()+1)
	EndIf
	
	If ai2combo.Selecteditem() <> -1 And ai2combo.SelectedItem() < 3
		AI.Make(ai2combo.SelectedItem()+1,ai2teamcombo.SelectedItem()+1)
	EndIf
	
	If ai3combo.Selecteditem() <> -1 And ai3combo.SelectedItem() < 3
		AI.Make(ai3combo.SelectedItem()+1,ai3teamcombo.SelectedItem()+1)
	EndIf
	
	If ai4combo.Selecteditem() <> -1 And ai4combo.SelectedItem() < 3
		AI.Make(ai4combo.SelectedItem()+1,ai4teamcombo.SelectedItem()+1)
	EndIf
	
	If ai5combo.Selecteditem() <> -1 And ai5combo.SelectedItem() < 3
		AI.Make(ai5combo.SelectedItem()+1,ai5teamcombo.SelectedItem()+1)
	EndIf
	
	Select gameTypeCombo.SelectedItem()
		Case 0
			MatchType = DEATHMATCH
	EndSelect		
	
	MatchPoint = Int(scoreLimitEdit.GetText())
	MatchTime = Int(timeLimitedit.GetText())*60
	MatchRespawn = Int(penaltyEdit.GetText())
	MatchType = Int(gameTypeCombo.selectedItem())
	
	If MatchType = 1
		tempVal = Rnd(1,AI.List.Count()+1)
		For Local temp:AI = EachIn AI.List
			x:+1
			If x = tempVal
				temp.iAmBigDog = 1
			EndIf
		Next
	EndIf

EndFunction

Function FindHighest:Int(Array:Int[])
	Local temp:Int
	
	For x = 0 To Array.Length-1
		If temp < Array[x]
			temp = Array[x]
		EndIf
	Next
	
	Return temp
EndFunction

Function LoadSFX()
	mgFire = LoadSound("SFX/machine_gun_fire.wav",0)
	hmgFire = LoadSound("SFX/heavy_machine_gun_fire.wav",0)
	glFire = LoadSound("SFX/grenade_launcher_fire.wav",0)
	rlFire = LoadSound("SFX/rocket_launcher_fire.wav",0)
	explode = LoadSound("SFX/explosion.wav",0)
	phaseFire = LoadSound("SFX/laser_fire.wav",0)
EndFunction