Rem
	Demolition Derby Explosion Type
EndRem

Type Explosion
	
	Field X:Float
	Field Y:Float
	Field Size:Int
	Field angDelta
	Field Angle
	Field scaleDelta:Float
	Field Scale:Float
	Field Life
	Field Parent:ControlObject
	
	Global Image:Timage
	Global List:TList
	
	Function Make:Explosion(X,Y,Size:Int=0,Parent:ControlObject=Null)
	
		Local tExp:Explosion = New Explosion
		If List = Null List = CreateList()
		List.AddLast tExp
		
		If Image = Null
			Image = LoadImage("sprites/explosion.png")
			MidHandleImage(Image)
		EndIf
		
		tExp.X = X
		tExp.Y = Y
		tExp.Parent = Parent
		tExp.Size = Size
		
		Select Size
			Case 3
				tExp.Scale = 0.75
			Case 2
				tExp.Scale = 0.50
			Case 1
				tExp.Scale = 0.25
			Case 0
				tExp.Scale = 0.05
		EndSelect
		
		If Size > 0 Then PlaySound(explode,explodeChannel)
		
		'tExp.Scale = 1
		tExp.scaleDelta = 0.007
		
		Local angDir:Int = Rnd(0,2)
		If angDir = 0
			tExp.angDelta = 2
		Else
			tExp.angDelta = -2
		EndIf
		
		If Size > 0
			tExp.Life = 50
		Else
			tExp.Life = 15
		EndIf
		
		Return tExp
	EndFunction
	
	Method Draw()
		SetScale Scale,Scale
		SetRotation Angle
		SetBlend ALPHABLEND
		SetAlpha 0.8
		DrawImage Image,X-px,Y-py
		SetAlpha 1
		SetScale 1,1
		SetRotation 0
	EndMethod
	
	Method Update()
		Draw()
		
		Angle:+angDelta
		If Size > 0
			Scale:+scaleDelta
		Else
			Scale:+0.0005
		EndIf
		
		Life:-1
		
		Collision()
		
		If Life = 1 And Size > 0 And GRAPHIC_QUALITY = GFX_HIGH
			Scar.Make(X,Y,Size)
		EndIf
		
		If Life <= 0
			Destroy()
		EndIf
	EndMethod
	
	Method Collision()
	
		Local Radius:Int
		Local DamageCap:Float
		Local tempDamage:Float
		
		Select Size
			Case 3
				Radius = 120
				DamageCap = 120
			Case 2
				Radius = 80
				DamageCap = 80
			Case 1
				Radius = 50
				DamageCap = 50
		EndSelect
		
		Local tVehicle:Vehicle
		For tVehicle = EachIn Vehicle.List
			If Distance(X,Y,tVehicle.X,tVehicle.Y) < Radius And tVehicle.Shield = False
				If tVehicle.Bonus = EXPLODE_PROTECT
					tempDamage = (DamageCap - Distance(X,Y,tVehicle.X,tVehicle.Y))/2
					Print tempDamage/30
				Else
					tempDamage = DamageCap - Distance(X,Y,tVehicle.X,tVehicle.Y)
				EndIf
				tVehicle.Parent.lastHit = Parent
				tVehicle.Health:-(tempDamage/30)
			EndIf
		Next
		
		If Mine.List 
		Local tMine:Mine
		For tMine = EachIn Mine.List
			If Distance(X,Y,tMine.X,tMine.Y) < Radius
				tMine.Destroy()
			EndIf
		Next
		EndIf
		
		If Shield.List
			Local tShield:Shield
			For tShield = EachIn Shield.List
				If Distance(X,Y,tShield.X,tShield.Y) < Radius
					tempDamage = DamageCap - Distance(X,Y,tShield.X,tShield.Y)
					tShield.Health:-(tempDamage/30)
				EndIf
			Next
		EndIf
		
	EndMethod
	
	Method Destroy()
		List.Remove(Self)
	EndMethod
	
	Function UpdateAll()
	
		If Not List Return
		
		Local tExp:Explosion
		For tExp = EachIn List
			tExp.Update()
		Next
	EndFunction
	
	Function DestroyAll()
		If Not List Return
		
		Local tExp:Explosion
		For tExp = EachIn List
			tExp.Destroy()
		Next
	EndFunction
	
EndType

Rem
	Scar Mark for explosions :) Quite handy to have
EndRem

Type Scar
	
	Field X:Float
	Field Y:Float
	Field Size:Int
	Field Scale:Float
	Field Angle:Float
	
	Global Image:Timage
	Global List:TList
	
	Function Make:Scar(X,Y,Size:Int=0)
		Local tExp:Scar = New Scar
		If List = Null List = CreateList()
		List.AddLast tExp
		
		If Image = Null
			Image = LoadImage("sprites/scar.png")
			MidHandleImage(Image)
		EndIf
		
		tExp.X = X
		tExp.Y = Y
		tExp.Size = Size
		tExp.Angle = Rnd(360)
		
		Select Size
			Case 3
				tExp.Scale = 0.75
			Case 2
				tExp.Scale = 0.50
			Case 1
				tExp.Scale = 0.25
			Case 0
				tExp.Scale = 0.05
		EndSelect
		
		Return tExp
	EndFunction
	
	Method Draw()
		SetScale Scale,Scale
		SetRotation Angle
		SetBlend ALPHABLEND
		SetAlpha 0.8
		DrawImage Image,X-px,Y-py
		SetAlpha 1
		SetScale 1,1
		SetRotation 0
	EndMethod
	
	Method Update()
		Draw()
	EndMethod
	
	Method Destroy()
		List.Remove(Self)
	EndMethod
	
	Function UpdateAll()
	
		If Not List Return
		
		Local tExp:Scar
		For tExp = EachIn List
			tExp.Update()
		Next
	EndFunction
	
	Function DestroyAll()
		If Not List Return
		
		Local tExp:Scar
		For tExp = EachIn List
			tExp.Destroy()
		Next
	EndFunction
	
EndType

	
