Rem
	Demolition Derby: Used Car Edition
	Weapon Type
EndRem

Type Weapon Extends GameObject
	
	Field RateOfFire:Int
	Field Ammo:Int
	Field BulletImage:TImage
	Field Parent:Vehicle
	Field Angle:Double
	Field Name:String
	Field gType:Int
	Field bulletLife:Int
	Field bulletSpeed:Int
	Field bulletSpins:Int
	Field bulletType:Int
	Field maxAmmo:Int
	Field playerControlled:Int
	Field Image:TImage
	Field Emitter:SpriteEmitter = SpriteEmitter.Create(plumeImage)
	Field ShotType:Shot
	Field Number:Int
	Global List:TList
	
	Function Make:Weapon(Parent:Vehicle)
	EndFunction
	
	Method Draw()
	
		SetBlend ALPHABLEND
		SetAlpha 255
		SetRotation Angle
		SetScale 0.25,0.25
		DrawImage(Image,X-px,Y-py)
		SetScale 1,1 
		SetRotation 0
		
	EndMethod
	
	Method Update()
	EndMethod
	
	Function UpdateAll()
		If MachineGun.List Then MachineGun.UpdateAll()
		If HeavyMachineGun.List Then HeavyMachineGun.UpdateAll()
		If RPG.List Then RPG.UpdateAll()
		If RocketLauncher.List Then RocketLauncher.UpdateAll()
		If PhaseGun.List Then PhaseGun.UpdateAll()
	EndFunction
	
	Method Destroy()
		List.Remove(Self)
	EndMethod

EndType

Type MachineGun Extends Weapon
	
	Function Choose:MachineGun(Parent:Vehicle)
	
		Local tMG:MachineGun = New MachineGun
		If List = Null List = CreateList()
		
		List.AddLast tMG
		
		tMG.Image = LoadImage("sprites/machinegun.png")
		MidHandleImage(tMG.Image)
		SetImageHandle(tMG.Image,36,24)
		
		tMG.Name = "Machine Gun"
		tMG.RateOfFire = 8
		tMG.Ammo = -1
		tMG.bulletLife = 800
		tMG.bulletSpeed = 25
		tMG.bulletSpins = False
		tMG.bulletType = 1
		tMG.maxAmmo = 0
		tMG.Parent = Parent
		tMG.ShotType = New MGBullet
		tMG.Number = 1
		
		tMG.Emitter.SetEmitCount(1)
		tMG.Emitter.SetStartAngle(90)
		tMG.Emitter.SetEndAngle(110)
		tMG.Emitter.SetEmitDelay(0.07)
		tMG.Emitter.SetEmitVelocity(2)
		tMG.Emitter.SetBlend(LIGHTBLEND)
		tMG.Emitter.SetAngleDelta(3)
		tMG.Emitter.SetScale(.25,.25)
		tMG.Emitter.SetAlphaDelta(0.05)	
		
		Return tMG
	EndFunction
	
	Method Update()
		Draw()
		
		
		' Check if the player is controlling the parent vehicle of the weapon.
		' Since the Sprite type handles objects differently, the weapon is placed
		' at a different location than if the object is player controlled.
		If Parent.PlayerControlled = 1
			X = Parent.X+(-5*Cos(Parent.Angle))
			Y = Parent.Y+(-5*Sin(Parent.Angle))
		Else
			X = Parent.X
			Y = Parent.Y
		EndIf
		
		If Parent.PlayerControlled = True
			Local tempX = MouseX()-(X-px)
			Local tempY = MouseY()-(Y-py)
			Angle = ATan2(tempY,tempX)
		Else
			tempX = 0
			tempY = 0
		EndIf
		
		If GRAPHIC_QUALITY => GFX_MED
			Emitter.SetCoordOffsets(px,py)
			Emitter.SetPosition(X+(35*Cos(Angle)),Y+(35*Sin(Angle)))
			Emitter.Update(Delta.Time())
		EndIf

	EndMethod
		
	Function UpdateAll()
		
		If Not List Return
		
		Local tMG:MachineGun
		For tMG = EachIn List
			tMG.Update()
		Next
		
	EndFunction
	
EndType	

Type HeavyMachineGun Extends Weapon
	
	Function Choose:HeavyMachineGun(Parent:Vehicle)
		Local tHMG:HeavyMachineGun = New HeavyMachineGun
		If List = Null List = CreateList()
		
		List.AddLast tHMG
		
		tHMG.Image = LoadImage("sprites/heavymachinegun.png")
		MidHandleImage(tHMG.Image)
		SetImageHandle(tHMG.Image,36,24)
		
		tHMG.Name = "Heavy Machine Gun"
		tHMG.RateOfFire = 7
		tHMG.Ammo = 250
		tHMG.bulletLife = 800
		tHMG.bulletSpeed = 15
		tHMG.bulletSpins = False
		tHMG.bulletType = 1
		tHMG.maxAmmo = 250
		tHMG.Parent = Parent
		tHMG.ShotType = New HMGBullet
		tHMG.Number = 2	
		tHMG.Emitter.SetEmitCount(1)
		tHMG.Emitter.SetStartAngle(90)
		tHMG.Emitter.SetEndAngle(110)
		tHMG.Emitter.SetEmitDelay(0.12)
		tHMG.Emitter.SetEmitVelocity(2)
		tHMG.Emitter.SetBlend(LIGHTBLEND)
		tHMG.Emitter.SetAngleDelta(3)
		tHMG.Emitter.SetScale(.25,.25)
		tHMG.Emitter.SetAlphaDelta(0.05)
		
		If Parent.Bonus = EXTRA_PRIMARY
			tHMG.Ammo:+50
		EndIf	
		
		Return tHMG
	EndFunction
	
	Method Update()
		Draw()
	
		X = Parent.X+(-5*Cos(Parent.Angle))
		Y = Parent.Y+(-5*Sin(Parent.Angle))
		
		If GRAPHIC_QUALITY => GFX_MED
			Emitter.SetCoordOffsets(px,py)
			Emitter.SetPosition(X+(35*Cos(Angle)),Y+(35*Sin(Angle)))
			Emitter.Update(Delta.Time())
		EndIf
		
		If Parent.PlayerControlled = True
			Local tempX = MouseX()-(X-px)
			Local tempY = MouseY()-(Y-py)
		Else
			tempX = 0
			tempY = 0
		EndIf
		Angle = ATan2(tempY,tempX)
		
		If Ammo <= 0	
			Parent.WeaponSlot = MachineGun.Choose(Parent)
			Destroy()
		EndIf

	EndMethod
		
	Function UpdateAll()
		
		If Not List Return
		
		Local tHMG:HeavyMachineGun
		For tHMG = EachIn List
			tHMG.Update()
		Next
		
	EndFunction
	
EndType

Type RPG Extends Weapon
	
	Function Choose:RPG(Parent:Vehicle)
		Local tRPG:RPG = New RPG
		If List = Null List = CreateList()
		
		List.AddLast tRPG
		
		tRPG.Image = LoadImage("sprites/rpg.png")
		MidHandleImage(tRPG.Image)
		SetImageHandle(tRPG.Image,60,32)
		
		tRPG.Name = "RPG"
		tRPG.RateOfFire = 50
		tRPG.Ammo = 20
		tRPG.bulletLife = 100
		tRPG.bulletSpeed = 5
		tRPG.bulletSpins = True
		tRPG.bulletType = 2
		tRPG.maxAmmo = 20
		tRPG.Parent = Parent
		tRPG.ShotType = New RPGBullet
		tRPG.Number = 3
		
		If Parent.Bonus = EXTRA_PRIMARY
			tRPG.Ammo:+4
		EndIf	
		
		Return tRPG
	EndFunction
	
	Method Update()
		Draw()
	
		X = Parent.X+(-5*Cos(Parent.Angle))
		Y = Parent.Y+(-5*Sin(Parent.Angle))
		
		If Parent.PlayerControlled = True
			Local tempX = MouseX()-(X-px)
			Local tempY = MouseY()-(Y-py)
		Else
			tempX = 0
			tempY = 0
		EndIf
		Angle = ATan2(tempY,tempX)
		
		If Ammo <= 0	
			Parent.WeaponSlot = MachineGun.Choose(Parent)
			Destroy()
		EndIf
			

	EndMethod
		
	Function UpdateAll()
		
		If Not List Return
		
		Local tRPG:RPG
		For tRPG = EachIn List
			tRPG.Update()
		Next
		
	EndFunction
	
EndType

Type RocketLauncher Extends Weapon
	
	Function Choose:RocketLauncher(Parent:Vehicle)
		Local tRL:RocketLauncher = New RocketLauncher
		If List = Null List = CreateList()
		
		List.AddLast tRL
		
		tRL.Image = LoadImage("sprites/rocketlauncher.png")
		MidHandleImage(tRL.Image)
		SetImageHandle(tRL.Image,100,32)
		
		tRL.Name = "Rocket Launcher"
		tRL.RateOfFire = 75
		tRL.Ammo = 10
		tRL.bulletLife = 500
		tRL.bulletSpeed = 7
		tRL.bulletSpins = True
		tRL.maxAmmo = 10
		tRL.Parent = Parent
		tRL.ShotType = New RLBullet
		tRL.Number = 4
		
		If Parent.Bonus = EXTRA_PRIMARY
			tRL.Ammo:+2
		EndIf			
		
		Return tRL
	EndFunction
	
	Method Draw()
	
		SetBlend ALPHABLEND
		SetAlpha 255
		SetRotation Angle
		SetScale 0.25,0.25
		DrawImage(Image,X-px,Y-py)
		SetScale 1,1 
		SetRotation 0
		
	EndMethod
	
	Method Update()
		Draw()
	
		X = Parent.X+(-5*Cos(Parent.Angle))
		Y = Parent.Y+(-5*Sin(Parent.Angle))
		
		If Parent.PlayerControlled = True
			Local tempX = MouseX()-(X-px)
			Local tempY = MouseY()-(Y-py)
		Else
			tempX = 0
			tempY = 0
		EndIf
		Angle = ATan2(tempY,tempX)
		
		If Ammo = 0	
			Parent.WeaponSlot = MachineGun.Choose(Parent)
			Destroy()
		EndIf
			

	EndMethod
		
	Function UpdateAll()
		
		If Not List Return
		
		Local tRL:RocketLauncher
		For tRL = EachIn List
			tRL.Update()
		Next
		
	EndFunction
	
EndType

Type PhaseGun Extends Weapon
	
	Function Choose:PhaseGun(Parent:Vehicle)
		Local tPG:PhaseGun = New PhaseGun
		If List = Null List = CreateList()
		
		List.AddLast tPG
		
		tPG.Image = LoadImage("sprites/phasecannon.png")
		MidHandleImage(tPG.Image)
		SetImageHandle(tPG.Image,100,32)
		
		tPG.Name = "Phase Cannon"
		tPG.RateOfFire = 75
		tPG.Ammo = 7
		tPG.bulletLife = 750
		tPG.bulletSpeed = 9
		tPG.bulletSpins = True
		tPG.maxAmmo = 7
		tPG.Parent = Parent
		tPG.ShotType = New PGBullet
		tPG.Number = 5
		
		If Parent.Bonus = EXTRA_PRIMARY
			tPG.Ammo:+2
		EndIf			
		
		Return tPG
	EndFunction
	
	Method Update()
		Draw()
	
		X = Parent.X+(-5*Cos(Parent.Angle))
		Y = Parent.Y+(-5*Sin(Parent.Angle))
		
		If Parent.PlayerControlled = True
			Local tempX = MouseX()-(X-px)
			Local tempY = MouseY()-(Y-py)
		Else
			tempX = 0
			tempY = 0
		EndIf
		Angle = ATan2(tempY,tempX)
		
		If Ammo = 0	
			Parent.WeaponSlot = MachineGun.Choose(Parent)
			Destroy()
		EndIf
			

	EndMethod
		
	Function UpdateAll()
		
		If Not List Return
		
		Local tPG:PhaseGun
		For tPG = EachIn List
			tPG.Update()
		Next
		
	EndFunction	
	
EndType


